#pragma once

#include "Generic.h"

#define image_numberof_directory_entries 16

//Directory format
typedef struct
{
	UINT32 VirtualAddress;
	UINT32 Size;
} image_data_directory, *pimage_data_directory;

//DOS .EXE header
typedef struct
{
	UINT16 e_magic;                     // Magic number
	UINT16 e_cblp;                      // Bytes on last page of file
	UINT16 e_cp;                        // Pages in file
	UINT16 e_crlc;                      // Relocations
	UINT16 e_cparhdr;                   // Size of header in paragraphs
	UINT16 e_minalloc;                  // Minimum extra paragraphs needed
	UINT16 e_maxalloc;                  // Maximum extra paragraphs needed
	UINT16 e_ss;                        // Initial (relative) SS value
	UINT16 e_sp;                        // Initial SP value
	UINT16 e_csum;                      // Checksum
	UINT16 e_ip;                        // Initial IP value
	UINT16 e_cs;                        // Initial (relative) CS value
	UINT16 e_lfarlc;                    // File address of relocation table
	UINT16 e_ovno;                      // Overlay number
	UINT16 e_res[4];                    // Reserved words
	UINT16 e_oemid;                     // OEM identifier (for e_oeminfo)
	UINT16 e_oeminfo;                   // OEM information; e_oemid specific
	UINT16 e_res2[10];                  // Reserved words
	INT32  e_lfanew;                    // File address of new exe header
} image_dos_header, *pimage_dos_header;

//Optional header format
typedef struct
{
	//Standard fields
	UINT16 Magic;
	UINT8  MajorLinkerVersion;
	UINT8  MinorLinkerVersion;
	UINT32 SizeOfCode;
	UINT32 SizeOfInitializedData;
	UINT32 SizeOfUninitializedData;
	UINT32 AddressOfEntryPoint;
	UINT32 BaseOfCode;
	UINT32 BaseOfData;

	//NT additional fields
	UINT32 ImageBase;
	UINT32 SectionAlignment;
	UINT32 FileAlignment;
	UINT16 MajorOperatingSystemVersion;
	UINT16 MinorOperatingSystemVersion;
	UINT16 MajorImageVersion;
	UINT16 MinorImageVersion;
	UINT16 MajorSubsystemVersion;
	UINT16 MinorSubsystemVersion;
	UINT32 Win32VersionValue;
	UINT32 SizeOfImage;
	UINT32 SizeOfHeaders;
	UINT32 CheckSum;
	UINT16 Subsystem;
	UINT16 DllCharacteristics;
	UINT32 SizeOfStackReserve;
	UINT32 SizeOfStackCommit;
	UINT32 SizeOfHeapReserve;
	UINT32 SizeOfHeapCommit;
	UINT32 LoaderFlags;
	UINT32 NumberOfRvaAndSizes;
	image_data_directory DataDirectory[image_numberof_directory_entries];
} image_optional_header32, *pimage_optional_header32;

typedef struct
{
	UINT16 Magic;
	UINT8  MajorLinkerVersion;
	UINT8  MinorLinkerVersion;
	UINT32 SizeOfCode;
	UINT32 SizeOfInitializedData;
	UINT32 SizeOfUninitializedData;
	UINT32 AddressOfEntryPoint;
	UINT32 BaseOfCode;
	UINT64 ImageBase;
	UINT32 SectionAlignment;
	UINT32 FileAlignment;
	UINT16 MajorOperatingSystemVersion;
	UINT16 MinorOperatingSystemVersion;
	UINT16 MajorImageVersion;
	UINT16 MinorImageVersion;
	UINT16 MajorSubsystemVersion;
	UINT16 MinorSubsystemVersion;
	UINT32 Win32VersionValue;
	UINT32 SizeOfImage;
	UINT32 SizeOfHeaders;
	UINT32 CheckSum;
	UINT16 Subsystem;
	UINT16 DllCharacteristics;
	UINT64 SizeOfStackReserve;
	UINT64 SizeOfStackCommit;
	UINT64 SizeOfHeapReserve;
	UINT64 SizeOfHeapCommit;
	UINT32 LoaderFlags;
	UINT32 NumberOfRvaAndSizes;
	image_data_directory DataDirectory[image_numberof_directory_entries];
} image_optional_header64, *pimage_optional_header64;

typedef struct
{
	UINT16 Machine;
	UINT16 NumberOfSections;
	UINT32 TimeDateStamp;
	UINT32 PointerToSymbolTable;
	UINT32 NumberOfSymbols;
	UINT16 SizeOfOptionalHeader;
	UINT16 Characteristics;
} image_file_header, *pimage_file_header;

typedef struct
{
	UINT32 Signature;
	image_file_header FileHeader;
	image_optional_header64 OptionalHeader;
} image_nt_headers64, *pimage_nt_headers64;

typedef struct
{
	UINT32 Signature;
	image_file_header FileHeader;
	image_optional_header32 OptionalHeader;
} image_nt_headers32, *pimage_nt_headers32;

//Section header format
typedef struct
{
	UINT8 Name[8];
	union
	{
		UINT32 PhysicalAddress;
		UINT32 VirtualSize;
	} Misc;

	UINT32 VirtualAddress;
	UINT32 SizeOfRawData;
	UINT32 PointerToRawData;
	UINT32 PointerToRelocations;
	UINT32 PointerToLinenumbers;
	UINT16 NumberOfRelocations;
	UINT16 NumberOfLinenumbers;
	UINT32 Characteristics;
} image_section_header, *pimage_section_header;

UINT32 image_scn_lnk_nreloc_ovfl = 0x01000000;          // Section contains extended relocations.
UINT32 image_scn_mem_discardable = 0x02000000;          // Section can be discarded.
UINT32 image_scn_mem_not_cached = 0x04000000;           // Section is not cachable.
UINT32 image_scn_mem_not_paged = 0x08000000;            // Section is not pageable.
UINT32 image_scn_mem_shared = 0x10000000;               // Section is shareable.
UINT32 image_scn_mem_execute = 0x20000000;              // Section is executable.
UINT32 image_scn_mem_read = 0x40000000;                 // Section is readable.
UINT32 image_scn_mem_write = 0x80000000;                // Section is writeable.

UINT32 image_scn_cnt_code = 0x00000020;                 // Section contains code.
UINT32 image_scn_cnt_initialized_data = 0x00000040;     // Section contains initialized data.
UINT32 image_scn_cnt_uninitialized_data = 0x00000080;   // Section contains uninitialized data.

UINT32 image_nt_optional_hdr32_magic = 0x10b;
UINT32 image_nt_optional_hdr64_magic = 0x20b;

