#include "SectList.h"

void InitList(PLIST_ENTRY Head)
{
    InitializeListHead(Head);
}

pSectionList FindEntry(PLIST_ENTRY Head, UINT32 Pid, PVOID BaseAddr, UINT32 RVA, const PUINT8 Name)
{
    PLIST_ENTRY Entry = NULL;
    pSectionList Section = NULL;
    
    for(Entry = Head->Flink; Head != Entry; Entry = Entry->Flink)
    {
        Section = CONTAINING_RECORD(Entry, SectionList, Next);
        
        if
        (
            Section->Pid == Pid
            &&
            Section->BaseAddr == BaseAddr
            &&
            Section->RVA == RVA
            &&
            strncmp( (const char *) Section->Name, (const char *) Name, sizeof(Section->Name)) == 0
        )
        {
            return Section;
        }
    }
    
    return NULL;
}

void AddEntry(PLIST_ENTRY Head, UINT32 Pid, PVOID BaseAddr, UINT32 RVA, const PUINT8 Name, UINT32 CRC32)
{
    pSectionList Node;
    
    Node = (pSectionList) kAlloc(sizeof(SectionList));
    NT_ASSERT(Node);
    
    RtlZeroMemory(Node, sizeof(SectionList));
    
    Node->Pid = Pid;
    Node->BaseAddr = BaseAddr;
    Node->CRC32 = CRC32;
    Node->RVA = RVA;
    Node->IsPresent = 1;
    strncpy( (char *) Node->Name, (char *) Name, 8 );
    
    
    InsertHeadList(Head, &Node->Next);
}

void DeleteEntry(PLIST_ENTRY Head, UINT32 Pid, PVOID BaseAddr, UINT32 RVA, const PUINT8 Name, UINT32 CRC32)
{
    PLIST_ENTRY Entry = NULL;
    pSectionList Section = NULL;
    
    for(Entry = Head->Flink; Head != Entry; Entry = Entry->Flink)
    {
        Section = CONTAINING_RECORD(Entry, SectionList, Next);
        
        if
        (
            (Pid == 0 || Section->Pid == Pid)
            &&
            (BaseAddr == 0 || Section->BaseAddr == BaseAddr)
            &&
            (Name == NULL || strncmp( (const char *) Section->Name, (const char *) Name, sizeof(Section->Name)) == 0)
            &&
            (CRC32 == 0 || Section->CRC32 == CRC32)
            &&
            (RVA == 0 || Section->RVA == RVA)
        )
        {
            RemoveEntryList(Entry);
            kFree(Section);
            break;
        }
    }
}

void CleanList(PLIST_ENTRY Head)
{
    PLIST_ENTRY Item = NULL;
    pSectionList Section = NULL;
    
    while(!IsListEmpty(Head))
    {
        Item = RemoveHeadList(Head);
        Section = CONTAINING_RECORD(Item, SectionList, Next);
        kFree(Section);
    }  
}

void ShowList(PLIST_ENTRY Head)
{
    PLIST_ENTRY Entry = NULL;
    pSectionList Section = NULL;

    for(Entry = Head->Flink; Head != Entry; Entry = Entry->Flink)
    {
        Section = CONTAINING_RECORD(Entry, SectionList, Next);
        CKSMLOG("(%d) PID=%u, BaseAddr=%08X, RVA=%08X, Name=%.8s, CRC32=%08X", __LINE__, Section->Pid, Section->BaseAddr, Section->RVA, Section->Name, Section->CRC32);
    }
}

void DeleteUnusedEntries(PLIST_ENTRY Head, UINT32 Pid)
{
    PLIST_ENTRY Entry = NULL;
    pSectionList Section = NULL;
    
    for(Entry = Head->Flink; Head != Entry; Entry = Entry->Flink)
    {
        Section = CONTAINING_RECORD(Entry, SectionList, Next);
        
        if(Section->Pid == Pid && !Section->IsPresent)
        {
            CKSMLOG("(%d) deleted: PID=%u, BaseAddr=%08X, RVA=%08X, Name=%.8s, CRC32=%08X", __LINE__, Section->Pid, Section->BaseAddr, Section->RVA, Section->Name, Section->CRC32);
            
            RemoveEntryList(Entry);
            kFree(Section);
        }
    }
}

void ZeroIsPresent(PLIST_ENTRY Head, UINT32 Pid)
{
    PLIST_ENTRY Entry = NULL;
    pSectionList Section = NULL;
    
    for(Entry = Head->Flink; Head != Entry; Entry = Entry->Flink)
    {
        Section = CONTAINING_RECORD(Entry, SectionList, Next);
        
        if(Section->Pid == Pid)
            Section->IsPresent = 0;
    }
}
