#include "..\ChecksumLib\ChecksumFunc.h"

typedef struct _SampleDeviceExt
{
    PDEVICE_OBJECT Fdo;
    UNICODE_STRING Symlink;
    KMUTEX Mutex;
} SAMPLE_DEVICE_EXT, *PSAMPLE_DEVICE_EXT;


#define DEV_NAME L"SampleDriver"
#define SYMLINK_NAME L"\\??\\" DEV_NAME
#define IOCTL_SAMPLE CTL_CODE(FILE_DEVICE_UNKNOWN, 0x801, METHOD_BUFFERED, FILE_ANY_ACCESS)

#pragma code_seg("INIT")

VOID DrvUnload(PDRIVER_OBJECT pDriverObject);
NTSTATUS DispatchRoutine(PDEVICE_OBJECT DeviceObject, PIRP Irp);

NTSTATUS DriverEntry(PDRIVER_OBJECT DriverObject, PUNICODE_STRING RegistryPath)
{
    NTSTATUS Status = STATUS_SUCCESS;
    PDEVICE_OBJECT Fdo;
    UNICODE_STRING DevName;
    PSAMPLE_DEVICE_EXT Pdx;
    UNICODE_STRING Symlink;
    UINT16 i;

    DbgPrint("(%d) DriverEntry", __LINE__);
    
    for (i = 0; i < IRP_MJ_MAXIMUM_FUNCTION; i++)
        DriverObject->MajorFunction[i] = DispatchRoutine;

    DriverObject->MajorFunction[i] = NULL;
    DriverObject->DriverUnload = DrvUnload;

    RtlInitUnicodeString(&DevName, L"\\Device\\" DEV_NAME);

    Status = IoCreateDevice
    (
        DriverObject,
        sizeof(SAMPLE_DEVICE_EXT),
        &DevName,
        FILE_DEVICE_UNKNOWN,
        0,
        FALSE,
        &Fdo
    );

    if(!NT_SUCCESS(Status))
        return Status;

    Pdx = (PSAMPLE_DEVICE_EXT) Fdo->DeviceExtension;
    Pdx->Fdo = Fdo;

    DbgPrint("(%d) FDO=%08X, DevExt=%X", __LINE__, Fdo, Pdx);

    RtlInitUnicodeString(&Symlink, SYMLINK_NAME);
    Pdx->Symlink = Symlink;

    Status = IoCreateSymbolicLink(&Symlink, &DevName);
    if(!NT_SUCCESS(Status))
    {
        IoDeleteDevice(Fdo);
        return Status;
    }

    
    KeInitializeMutex(&Pdx->Mutex, 0);
    ChecksumInit();

    DbgPrint("(%d) Driver loaded", __LINE__);

    return Status;
}

#pragma code_seg()

NTSTATUS DispatchRoutine(PDEVICE_OBJECT DeviceObject, PIRP Irp)
{
    PSAMPLE_DEVICE_EXT Pdx;
    PIO_STACK_LOCATION pIrpStack = IoGetCurrentIrpStackLocation(Irp);
    Irp->IoStatus.Information = 0;
    Irp->IoStatus.Status = STATUS_SUCCESS;

    Pdx = (PSAMPLE_DEVICE_EXT) DeviceObject->DeviceExtension;

    DbgPrint("(%d) DispatchRoutine", __LINE__);

    KeWaitForMutexObject(&Pdx->Mutex, UserRequest, KernelMode, FALSE, NULL);

    switch (pIrpStack->MajorFunction)
    {
        case IRP_MJ_CREATE:
            DbgPrint("(%d) IRP_MJ_CREATE", __LINE__);
        break;

        case IRP_MJ_CLOSE:
            DbgPrint("(%d) IRP_MJ_CLOSE", __LINE__);
        break;

        case IRP_MJ_DEVICE_CONTROL:
            DbgPrint("(%d) IRP_MJ_DEVICE_CONTROL", __LINE__);
            switch (pIrpStack->Parameters.DeviceIoControl.IoControlCode) 
            {
                case IOCTL_SAMPLE:
                {
                    BOOLEAN Status;
                    UINT32 OutSize = pIrpStack->Parameters.DeviceIoControl.OutputBufferLength;

                    if(OutSize != sizeof(BOOLEAN))
                    {
                        Irp->IoStatus.Status = STATUS_INVALID_PARAMETER;
                        break;
                    }

                    Status = CheckUserProcess();
                    
                    RtlCopyMemory(Irp->AssociatedIrp.SystemBuffer, &Status, sizeof(BOOLEAN));
                    Irp->IoStatus.Information = sizeof(BOOLEAN);
                }
                break;
            }
        break;

        default:
            DbgPrint("(%d) Not implemented major function call", __LINE__);
            Irp->IoStatus.Status = STATUS_NOT_IMPLEMENTED;
        break;
    }

    KeReleaseMutex(&Pdx->Mutex, FALSE);

    IoCompleteRequest(Irp, IO_NO_INCREMENT);

    return Irp->IoStatus.Status;
}

VOID DrvUnload(PDRIVER_OBJECT pDriverObject)
{
    PDEVICE_OBJECT NextDevObj;
    int i;

    DbgPrint("(%d) DrvUnload", __LINE__);

    NextDevObj = pDriverObject->DeviceObject;

    for(i = 0; NextDevObj != NULL; i++)
    {
        PSAMPLE_DEVICE_EXT Dx = (PSAMPLE_DEVICE_EXT) NextDevObj->DeviceExtension;
        UNICODE_STRING * Link = &(Dx->Symlink);

        NextDevObj = NextDevObj->NextDevice;

        IoDeleteSymbolicLink(Link);
        IoDeleteDevice(Dx->Fdo);
    }

    ChecksumFinalize();
}
